const winston = require('winston');
const InMemoryAdapter = require('@windingtree/off-chain-adapter-in-memory');

module.exports = {
  port: 8100,
  baseUrl: 'http://example.com',
  entrypointAddress: 'will-be-set-during-init',
  ethNetwork: 'test',
  checkTrustClues: true,
  wtLibsOptions: {
    onChainDataOptions: {
      provider: 'http://localhost:8545',
    },
    offChainDataOptions: {
      adapters: {
        'in-memory': {
          options: { },
          create: (options) => {
            return new InMemoryAdapter(options);
          },
        },
      },
    },
    trustClueOptions: {
      provider: 'http://localhost:8545',
      clues: {
        'test-list': {
          create: async (options) => {
            return {
              getMetadata: () => ({
                name: 'test-list',
                description: '',
              }),
              getValueFor: (addr) => {
                return true;
              },
              interpretValueFor: (addr) => {
                return true;
              },
            };
          },
        },
      },
    },
  },
  logger: winston.createLogger({
    level: 'warn',
    transports: [
      new winston.transports.Console({
        format: winston.format.simple(),
        handleExceptions: true,
      }),
    ],
  }),
};
